"use strict";
exports.__esModule = true;
var loaded_1 = require("./loaded");
console.log(loaded_1["default"](0));
console.log(loaded_1["default"](1));
console.log(loaded_1["default"](-1));
//# sourceMappingURL=main.js.map