"use strict";
exports.__esModule = true;
function Unloaded() {
    return 'Never loaded :(';
}
exports["default"] = Unloaded;
console.log("This file shouldn't have been evaluated");
//# sourceMappingURL=unloaded.js.map