function cov_2p08rjaqat() {
  var path = "/Users/bencoe/oss/c8/test/fixtures/source-maps/classes/classes.js";
  var hash = "74ac795d8a024884bc2228cec55d8411c5de3312";
  var global = new Function("return this")();
  var gcv = "__coverage__";
  var coverageData = {
    path: "/Users/bencoe/oss/c8/test/fixtures/source-maps/classes/classes.js",
    statementMap: {
      "0": {
        start: {
          line: 3,
          column: 4
        },
        end: {
          line: 3,
          column: 23
        }
      },
      "1": {
        start: {
          line: 4,
          column: 4
        },
        end: {
          line: 8,
          column: 5
        }
      },
      "2": {
        start: {
          line: 5,
          column: 6
        },
        end: {
          line: 5,
          column: 29
        }
      },
      "3": {
        start: {
          line: 7,
          column: 6
        },
        end: {
          line: 7,
          column: 31
        }
      },
      "4": {
        start: {
          line: 9,
          column: 4
        },
        end: {
          line: 9,
          column: 18
        }
      },
      "5": {
        start: {
          line: 12,
          column: 4
        },
        end: {
          line: 12,
          column: 27
        }
      },
      "6": {
        start: {
          line: 15,
          column: 4
        },
        end: {
          line: 15,
          column: 29
        }
      },
      "7": {
        start: {
          line: 18,
          column: 4
        },
        end: {
          line: 18,
          column: 27
        }
      },
      "8": {
        start: {
          line: 21,
          column: 4
        },
        end: {
          line: 21,
          column: 29
        }
      },
      "9": {
        start: {
          line: 25,
          column: 10
        },
        end: {
          line: 25,
          column: 20
        }
      },
      "10": {
        start: {
          line: 26,
          column: 10
        },
        end: {
          line: 26,
          column: 21
        }
      },
      "11": {
        start: {
          line: 27,
          column: 0
        },
        end: {
          line: 27,
          column: 11
        }
      }
    },
    fnMap: {
      "0": {
        name: "(anonymous_0)",
        decl: {
          start: {
            line: 2,
            column: 2
          },
          end: {
            line: 2,
            column: 3
          }
        },
        loc: {
          start: {
            line: 2,
            column: 21
          },
          end: {
            line: 10,
            column: 3
          }
        },
        line: 2
      },
      "1": {
        name: "(anonymous_1)",
        decl: {
          start: {
            line: 11,
            column: 2
          },
          end: {
            line: 11,
            column: 3
          }
        },
        loc: {
          start: {
            line: 11,
            column: 13
          },
          end: {
            line: 13,
            column: 3
          }
        },
        line: 11
      },
      "2": {
        name: "(anonymous_2)",
        decl: {
          start: {
            line: 14,
            column: 2
          },
          end: {
            line: 14,
            column: 3
          }
        },
        loc: {
          start: {
            line: 14,
            column: 13
          },
          end: {
            line: 16,
            column: 3
          }
        },
        line: 14
      },
      "3": {
        name: "(anonymous_3)",
        decl: {
          start: {
            line: 17,
            column: 2
          },
          end: {
            line: 17,
            column: 3
          }
        },
        loc: {
          start: {
            line: 17,
            column: 13
          },
          end: {
            line: 19,
            column: 3
          }
        },
        line: 17
      },
      "4": {
        name: "(anonymous_4)",
        decl: {
          start: {
            line: 20,
            column: 2
          },
          end: {
            line: 20,
            column: 3
          }
        },
        loc: {
          start: {
            line: 20,
            column: 13
          },
          end: {
            line: 22,
            column: 3
          }
        },
        line: 20
      }
    },
    branchMap: {
      "0": {
        loc: {
          start: {
            line: 2,
            column: 15
          },
          end: {
            line: 2,
            column: 19
          }
        },
        type: "default-arg",
        locations: [{
          start: {
            line: 2,
            column: 17
          },
          end: {
            line: 2,
            column: 19
          }
        }],
        line: 2
      },
      "1": {
        loc: {
          start: {
            line: 3,
            column: 13
          },
          end: {
            line: 3,
            column: 23
          }
        },
        type: "cond-expr",
        locations: [{
          start: {
            line: 3,
            column: 17
          },
          end: {
            line: 3,
            column: 18
          }
        }, {
          start: {
            line: 3,
            column: 21
          },
          end: {
            line: 3,
            column: 23
          }
        }],
        line: 3
      },
      "2": {
        loc: {
          start: {
            line: 4,
            column: 4
          },
          end: {
            line: 8,
            column: 5
          }
        },
        type: "if",
        locations: [{
          start: {
            line: 4,
            column: 4
          },
          end: {
            line: 8,
            column: 5
          }
        }, {
          start: {
            line: 4,
            column: 4
          },
          end: {
            line: 8,
            column: 5
          }
        }],
        line: 4
      }
    },
    s: {
      "0": 0,
      "1": 0,
      "2": 0,
      "3": 0,
      "4": 0,
      "5": 0,
      "6": 0,
      "7": 0,
      "8": 0,
      "9": 0,
      "10": 0,
      "11": 0
    },
    f: {
      "0": 0,
      "1": 0,
      "2": 0,
      "3": 0,
      "4": 0
    },
    b: {
      "0": [0],
      "1": [0, 0],
      "2": [0, 0]
    },
    _coverageSchema: "1a1c01bbd47fc00a2c39e90264f33305004495a9",
    hash: "74ac795d8a024884bc2228cec55d8411c5de3312"
  };
  var coverage = global[gcv] || (global[gcv] = {});

  if (!coverage[path] || coverage[path].hash !== hash) {
    coverage[path] = coverageData;
  }

  var actualCoverage = coverage[path];
  {
    // @ts-ignore
    cov_2p08rjaqat = function () {
      return actualCoverage;
    };
  }
  return actualCoverage;
}

cov_2p08rjaqat();

class Foo {
  constructor(x = (cov_2p08rjaqat().b[0][0]++, 33)) {
    cov_2p08rjaqat().f[0]++;
    cov_2p08rjaqat().s[0]++;
    this.x = x ? (cov_2p08rjaqat().b[1][0]++, x) : (cov_2p08rjaqat().b[1][1]++, 99);
    cov_2p08rjaqat().s[1]++;

    if (this.x) {
      cov_2p08rjaqat().b[2][0]++;
      cov_2p08rjaqat().s[2]++;
      console.info('covered');
    } else {
      cov_2p08rjaqat().b[2][1]++;
      cov_2p08rjaqat().s[3]++;
      console.info('uncovered');
    }

    cov_2p08rjaqat().s[4]++;
    this.methodC();
  }

  methodA() {
    cov_2p08rjaqat().f[1]++;
    cov_2p08rjaqat().s[5]++;
    console.info('covered');
  }

  methodB() {
    cov_2p08rjaqat().f[2]++;
    cov_2p08rjaqat().s[6]++;
    console.info('uncovered');
  }

  methodC() {
    cov_2p08rjaqat().f[3]++;
    cov_2p08rjaqat().s[7]++;
    console.info('covered');
  }

  methodD() {
    cov_2p08rjaqat().f[4]++;
    cov_2p08rjaqat().s[8]++;
    console.info('uncovered');
  }

}

const a = (cov_2p08rjaqat().s[9]++, new Foo(0));
const b = (cov_2p08rjaqat().s[10]++, new Foo(33));
cov_2p08rjaqat().s[11]++;
a.methodA();
//# sourceMappingURL=data:application/json;charset=utf-8;base64,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
