require('./async')

console.info('i am a line of code')

function apple (awesome) {
  if (false || true) {
    console.info('what')
  }
  if (true || false) {
    console.log('hey')
  }
}

function missed () {
  
}

function missed2 () {
  
}

apple()
apple()
apple()
